@@ -1,27 +1,26 @@
 '''
-mkfs.py - this file is part of S3QL (http://s3ql.googlecode.com)
+mkfs.py - this file is part of S3QL.
 
-Copyright (C) 2008-2009 Nikolaus Rath <Nikolaus@rath.org>
+Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, setup_logging, QuietError
-from . import CURRENT_FS_REV
-from .backends.common import get_backend, BetterBackend, DanglingStorageURLError
+from . import CURRENT_FS_REV, CTRL_INODE, ROOT_INODE
+from .backends.comprenc import ComprencBackend
 from .backends import s3
-from .common import get_backend_cachedir, CTRL_INODE, stream_write_bz2, PICKLE_PROTOCOL
+from .common import (get_backend_cachedir, get_backend, split_by_n,
+                     freeze_basic_mapping, time_ns)
 from .database import Connection
-from .metadata import dump_metadata, create_tables
+from .metadata import dump_and_upload_metadata, create_tables
 from .parse_args import ArgumentParser
 from getpass import getpass
-from llfuse import ROOT_INODE
+from base64 import b64encode
 import os
-import pickle
 import shutil
 import stat
 import sys
-import tempfile
 import time
 import atexit
 
@@ -34,12 +33,11 @@ def parse_args(args):
 
     parser.add_cachedir()
     parser.add_authfile()
-    parser.add_debug_modules()
+    parser.add_debug()
     parser.add_quiet()
-    parser.add_ssl()
+    parser.add_backend_options()
     parser.add_version()
     parser.add_storage_url()
-    parser.add_fatal_warnings()
 
     parser.add_argument("-L", default='', help="Filesystem label",
                       dest="label", metavar='<name>',)
@@ -58,24 +56,24 @@ def parse_args(args):
 
 def init_tables(conn):
     # Insert root directory
-    timestamp = time.time()
-    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime,atime,ctime,refcount) "
+    now_ns = time_ns()
+    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime_ns,atime_ns,ctime_ns,refcount) "
                  "VALUES (?,?,?,?,?,?,?,?)",
                    (ROOT_INODE, stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
                    | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH,
-                    os.getuid(), os.getgid(), timestamp, timestamp, timestamp, 1))
+                    os.getuid(), os.getgid(), now_ns, now_ns, now_ns, 1))
 
-    # Insert control inode, the actual values don't matter that much 
-    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime,atime,ctime,refcount) "
+    # Insert control inode, the actual values don't matter that much
+    conn.execute("INSERT INTO inodes (id,mode,uid,gid,mtime_ns,atime_ns,ctime_ns,refcount) "
                  "VALUES (?,?,?,?,?,?,?,?)",
                  (CTRL_INODE, stat.S_IFREG | stat.S_IRUSR | stat.S_IWUSR,
-                  0, 0, timestamp, timestamp, timestamp, 42))
+                  0, 0, now_ns, now_ns, now_ns, 42))
 
     # Insert lost+found directory
-    inode = conn.rowid("INSERT INTO inodes (mode,uid,gid,mtime,atime,ctime,refcount) "
+    inode = conn.rowid("INSERT INTO inodes (mode,uid,gid,mtime_ns,atime_ns,ctime_ns,refcount) "
                        "VALUES (?,?,?,?,?,?,?)",
                        (stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR,
-                        os.getuid(), os.getgid(), timestamp, timestamp, timestamp, 1))
+                        os.getuid(), os.getgid(), now_ns, now_ns, now_ns, 1))
     name_id = conn.rowid('INSERT INTO names (name, refcount) VALUES(?,?)',
                          (b'lost+found', 1))
     conn.execute("INSERT INTO contents (name_id, inode, parent_inode) VALUES(?,?,?)",
@@ -91,24 +89,20 @@ def main(args=None):
 
     if options.max_obj_size < 1024:
         # This warning should never be converrted to an exception
-        log.warning('Warning: maximum object sizes less than 1 MiB will seriously degrade '
-                 'performance.', extra={ 'force_log': True })
+        log.warning('Maximum object sizes less than 1 MiB will degrade '
+                    'performance.', extra={ 'force_log': True })
 
-    try:
-        plain_backend = get_backend(options, plain=True)
-        atexit.register(plain_backend.close)
-    except DanglingStorageURLError as exc:
-        raise QuietError(str(exc))
+    plain_backend = get_backend(options, raw=True)
+    atexit.register(plain_backend.close)
 
     log.info("Before using S3QL, make sure to read the user's guide, especially\n"
              "the 'Important Rules to Avoid Loosing Data' section.")
 
     if isinstance(plain_backend, s3.Backend) and '.' in plain_backend.bucket_name:
-        log.warning('***Warning*** S3 Buckets with names containing dots cannot be '
-                    'accessed using SSL!')
-        log.warning('(cf. https://forums.aws.amazon.com/thread.jspa?threadID=130560)')
+        log.warning('S3 Buckets with names containing dots cannot be '
+                    'accessed using SSL!'
+                    '(cf. https://forums.aws.amazon.com/thread.jspa?threadID=130560)')
 
-        
     if 's3ql_metadata' in plain_backend:
         if not options.force:
             raise QuietError("Found existing file system! Use --force to overwrite")
@@ -133,7 +127,7 @@ def main(args=None):
         data_pw = fh.read(32)
         fh.close()
 
-        backend = BetterBackend(wrap_pw, ('lzma', 2), plain_backend)
+        backend = ComprencBackend(wrap_pw, ('lzma', 2), plain_backend)
         backend['s3ql_passphrase'] = data_pw
         backend['s3ql_passphrase_bak1'] = data_pw
         backend['s3ql_passphrase_bak2'] = data_pw
@@ -141,10 +135,10 @@ def main(args=None):
     else:
         data_pw = None
 
-    backend = BetterBackend(data_pw, ('lzma', 2), plain_backend)
+    backend = ComprencBackend(data_pw, ('lzma', 2), plain_backend)
     atexit.unregister(plain_backend.close)
     atexit.register(backend.close)
-    
+
     # Setup database
     cachepath = get_backend_cachedir(options.storage_url, options.cachedir)
 
@@ -171,27 +165,20 @@ def main(args=None):
     param['last_fsck'] = time.time()
     param['last-modified'] = time.time()
 
-    # This indicates that the convert_legacy_metadata() stuff
-    # in BetterBackend is not required for this file system.
-    param['backend_revision'] = 1
-
     log.info('Dumping metadata...')
-    with tempfile.TemporaryFile() as fh:
-        dump_metadata(db, fh)
-        def do_write(obj_fh):
-            fh.seek(0)
-            stream_write_bz2(fh, obj_fh)
-            return obj_fh
-
-        log.info("Compressing and uploading metadata...")
-        backend.store('s3ql_seq_no_%d' % param['seq_no'], b'Empty')
-        obj_fh = backend.perform_write(do_write, "s3ql_metadata", metadata=param,
-                                      is_compressed=True)
-        
-    log.info('Wrote %.2f MiB of compressed metadata.', obj_fh.get_obj_size() / 1024 ** 2)
+    dump_and_upload_metadata(backend, db, param)
+    backend.store('s3ql_seq_no_%d' % param['seq_no'], b'Empty')
     with open(cachepath + '.params', 'wb') as fh:
-        pickle.dump(param, fh, PICKLE_PROTOCOL)
+        fh.write(freeze_basic_mapping(param))
 
+    if data_pw is not None:
+        print('Please store the following master key in a safe location. It allows ',
+              'decryption of the S3QL file system in case the storage objects holding ',
+              'this information get corrupted:',
+              '---BEGIN MASTER KEY---',
+              ' '.join(split_by_n(b64encode(data_pw).decode(), 4)),
+              '---END MASTER KEY---',
+              sep='\n')
 
 if __name__ == '__main__':
     main(sys.argv[1:])
